#include "dglwidget.h"

const int DGLWidget::OSD_TEXT_MS = (int)(1000 * 1.5f);
const int DGLWidget::ANIMATE_FPS = 30;
const QFont DGLWidget::OSD_FONT("sans-serif", 36);

#define MS_PER_FRAME (1000 / DGLWidget::ANIMATE_FPS)

DGLWidget::DGLWidget(QWidget *parent) :
        QGLWidget(parent)
{
    initialize();
}

DGLWidget::DGLWidget(QWidget *parent, const QGLFormat &format) :
        QGLWidget(parent)
{
    setFormat(format);
    initialize();
}

void DGLWidget::initialize()
{
    _osd.msleft = 0;
    _animatetimer = new QTimer(this);

    connect(_animatetimer, SIGNAL(timeout()), this, SLOT(animate()));
    _animatetimer->start(MS_PER_FRAME);
}

DGLWidget::~DGLWidget()
{
    delete _animatetimer;
}

void DGLWidget::resizeGL(int w, int h)
{
    _window_w = w; _window_h = h;
}

void DGLWidget::initializeGL()
{
    setAutoBufferSwap(false);
}

void DGLWidget::paintGL()
{
    // paint the OSD
    if (_osd.msleft > 0) {
        // white background
        glColor4f(1.0f, 1.0f, 1.0f, _osd.alpha/4.0f);
        renderText((width() - _osd.w)/2+1, (height() - _osd.h)/2+1, _osd.text, DGLWidget::OSD_FONT);
        renderText((width() - _osd.w)/2-1, (height() - _osd.h)/2+1, _osd.text, DGLWidget::OSD_FONT);
        renderText((width() - _osd.w)/2+1, (height() - _osd.h)/2-1, _osd.text, DGLWidget::OSD_FONT);
        renderText((width() - _osd.w)/2-1, (height() - _osd.h)/2-1, _osd.text, DGLWidget::OSD_FONT);
        // black foreground
        glColor4f(0.0f, 0.0f, 0.0f, _osd.alpha);
        renderText((width() - _osd.w)/2, (height() - _osd.h)/2, _osd.text, DGLWidget::OSD_FONT);
    }

    swapBuffers();
}

void DGLWidget::animate(int ms)
{
    if (_osd.msleft > 0) {
        _osd.msleft -= ms;
        if (_osd.msleft <= 0) {
            _osd.msleft = 0;
            emit OSDTextGone();
        }

        if (_osd.msleft * 2 >= DGLWidget::OSD_TEXT_MS)
            _osd.alpha = 1.0f; // full opacity for the first half of the time
        else
            _osd.alpha = (_osd.msleft * 2.0f) / DGLWidget::OSD_TEXT_MS; // fade for the last half
    }
}

void DGLWidget::animate()
{
    animate(MS_PER_FRAME);

    glDraw();
}

void DGLWidget::showOSDText(const QString &text)
{
    QFontMetrics m(DGLWidget::OSD_FONT);
    QRect r = m.boundingRect(text);

    _osd.text = text;
    _osd.w = r.width();
    _osd.h = r.height();
    _osd.msleft = DGLWidget::OSD_TEXT_MS;
}
