#ifndef DGLWIDGET_H
#define DGLWIDGET_H

#include <QGLWidget>
#include <QTimer>
#include <QFont>

class DGLWidget : public QGLWidget
{
    Q_OBJECT

protected:
    static const QFont OSD_FONT;  // the font used for osd text
    static const int ANIMATE_FPS; // the rate at which the gl display is updated
    static const int OSD_TEXT_MS; // how long osd text is displayed for

private:
    int _window_w;
    int _window_h;

    struct {
        QString text;
        int msleft;
        float alpha;
        int w;
        int h;
    } _osd;

    QTimer *_animatetimer;

public:
    DGLWidget(QWidget *parent);
    DGLWidget(QWidget *parent, const QGLFormat &format);
    ~DGLWidget();

    void showOSDText(const QString &text);
    bool isOSDTextVisble() const { return _osd.msleft > 0; }

signals:
    void OSDTextGone();

protected:
    int width() const { return _window_w; }
    int height() const { return _window_h; }

    virtual void initializeGL();
    virtual void resizeGL(int w, int h);
    virtual void paintGL();
    virtual void animate(int ms);

private:
    void initialize();

private slots:
    void animate();
};

#endif // DGLWIDGET_H
