#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QtGui/QMainWindow>
#include "chull.h"

namespace Ui
{
    class MainWindowClass;
}

class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    MainWindow(QWidget *parent = 0);
    ~MainWindow();

public slots:
    void toggleMaximized();

private:
    Ui::MainWindowClass *ui;
    PointSet *_set;
    CHull *_hull;
    bool _restart;
    bool _waiting_for_anim;
    QMutex _waitmutex;

    void setPointSet(class PointSet *p);
    void animationFinished();

private slots:
    void on_actionAbout_triggered();
    void on_actionUse_Default_Point_Set_triggered();
    void on_actionOpen_Point_Set_triggered();
    void on_autoZoom_clicked(bool checked);
    void on_actionRestart_triggered();
    void on_animationSpeed_valueChanged(int );
    void on_actionNext_Hull_triggered();
    void on_actionReset_3d_View_triggered();
    void on_actionNew_Point_Set_triggered();
    // events from the convex hull computation
    void doneLevel(bool bottom, int level);
    void updateHull(const Point *u, const Point *v);
    void initialHull(bool bottom, int level, const Point *u, const Point *v);
    void nextEvent(double t, const Point *p, const Point *q, const Point *r);
    // events fromthe visuals
    void projectionShowingTime(double t);
};

#endif // MAINWINDOW_H
