#include "pointset.h"
#include <stdlib.h>
#include <time.h>
#include <fstream>
#include <iostream>
#include <QFile>

static inline double ABS(double a) { return (a < 0 ? -a : a); }

PointSet* PointSet::generateDefault()
{
    PointSet *set = new PointSet;

#if 0
    Point *p = new Point[4];
    p[0].setPoint(-10,  0, 0);
    p[1].setPoint(-3, -3, 0);
    p[2].setPoint(7, 0, 2);
    p[3].setPoint(10, 3, 5);
    set->_size = 4;
#else
    Point *p = new Point[8];
    p[0].setPoint(-5,  -0.5, 1.5);
    p[1].setPoint(9.8, 0, 1);
    p[2].setPoint(-2.5, 10, -2.5);
    p[3].setPoint(10, 10.75, -2);
    p[4].setPoint(-2, 12, 12);
    p[5].setPoint(12.5, 11, 12.5);
    p[6].setPoint(0, -2, 9.5);
    p[7].setPoint(10.5, -1, 12.5);
    set->_size = 8;
#endif

    set->_points = p;
    set->colorize();

#ifdef DEBUG
    // double check the sort worked
    for (Point *pt = set->list(); pt && pt->next; pt = pt->next)
        assert(pt->x() <= pt->next->x());
#endif

    return set;
}

PointSet* PointSet::generate(int size)
{
    PointSet *set = 0;

    if (size >= 3) {
        set = new PointSet;

        // generate the points
        Point *p = new Point[size];
        for (int i = 0; i < size; ++i) {
            // generate points with coords between -500 and 500
            p[i].setPoint(lrand48() % 500000001 / (500000000.0 / 1000.0) - 500.0,
                          lrand48() % 500000001 / (500000000.0 / 1000.0) - 500.0,
                          lrand48() % 500000001 / (500000000.0 / 1000.0) - 500.0);
        }
        std::cout.flush();
        set->_size = size;

        set->_points = p;
        set->colorize();

#ifdef DEBUG
        // double check the sort worked
        for (Point *pt = set->list(); pt && pt->next; pt = pt->next)
            assert(pt->x() <= pt->next->x());
#endif
    }

    return set;
}

PointSet* PointSet::load(const QString &fname, QString **error)
{
    std::ifstream in;
    QByteArray encfname;

    encfname = QFile::encodeName(fname);
    in.open(encfname.constData(), std::ios::in);

    if (!in.is_open()) {
        *error = new QString("Unable to open the file for reading.");
        return 0;
    }

    int size, i;

    in >> size;
    if (in.eof() || size < 3) {
        *error = new QString("Invalid point set (bad size).");
        in.close();
        return 0;
    }

    Point *p = new Point[size];
    for (i = 0; i < size; ++i) {
        double x, y, z;
        in >> x;
        in >> y;
        in >> z;
        if (in.eof()) break;
        p[i].setPoint(x, y, z);
    }
    in.close();

    if (i < size) {
        delete p;
        *error = new QString("Invalid point set (missing points).");
        return 0;
    }

    PointSet *set = new PointSet();
    set->_size = size;
    set->_points = p;
    set->colorize();
    return set;
}

PointSet::PointSet()
{
    _points = 0;
    _size = 0;
    _ref = 1;
}

PointSet::~PointSet()
{
    delete _points;
}

void PointSet::ref()
{
    ++_ref;
}

void PointSet::unref()
{
    if (--_ref < 1)
        delete this;
}

Point *PointSet::sort()
{
    return sort(_points, _size);
}

Point* PointSet::sort(Point *P, unsigned long n)
{
    Point *in1, *in2, *merged, head;

    if (n == 1) { P[0].next = Point::NIL; return P; } // base case

    in1 = sort(P, n/2);
    in2 = sort(P+n/2, n-n/2);
    merged = &head;

    // merge in1 and in2 thru their 'next' pointers
    do {
        if (in1[0].x() < in2[0].x()) {
            // link merged to in1[0]
            merged = merged->next = &in1[0];
            in1 = in1->next;
        }
        else {
            // link merged to in2[0]
            merged = merged->next = &in2[0];
            in2 = in2->next;
        }
    } while (merged != Point::NIL);

    // return the first element in the merged list
    return head.next;
}

void PointSet::colorize()
{
    int i;

    srand(time(NULL));
    for (i = 0; i < _size; ++i) {
        _points[i].red = (rand() % 101) / 120.0;
        _points[i].blue = (rand() % 101) / 120.0;
        _points[i].green = (rand() % 101) / 120.0;
    }
}
