#ifndef POINTSETDISPLAY_H
#define POINTSETDISPLAY_H

#include "DGLWidget.h"

#include "pointset.h"
#include "face.h"

class PointSetDisplay : public DGLWidget
{
    Q_OBJECT

private:
    PointSet *_set;
    int _mousex;
    int _mousey;
    bool _translate;
    bool _zoom;
    bool _rotate;

    // the camera
    float _top;
    float _bottom;
    float _left;
    float _right;
    float _near;

    // view rotation
    float _rotx;
    float _roty;
    float _rotz;
    // view position
    float _posx;
    float _posy;
    float _posz;
    // center of the point set
    float _minx, _maxx;
    float _miny, _maxy;
    float _minz, _maxz;

    bool _viewinit;

    Face *_faces;
    int _nfaces;
    bool _hilightlast;

public:
    PointSetDisplay(QWidget *parent);
    virtual ~PointSetDisplay();

    void setPointSet(PointSet *set);
    void clearFaces();
    void addFace(const Point *p, const Point *q, const Point *r);
    void resetView();
    void highlightLastFace(bool h);

protected:
    void initializeGL();
    void resizeGL(int w, int h);
    void paintGL();
    void paintText();
    void initView();

    void mousePressEvent(QMouseEvent *e);
    void mouseReleaseEvent(QMouseEvent *e);
    void mouseMoveEvent(QMouseEvent *e);
};

#endif // POINTSETDISPLAY_H
